@@ -1,4 +1,4 @@
-#!/usr/bin/python2.4
+#!/usr/bin/python
 #
 # Copyright 2007 Google Inc.
 #
@@ -23,30 +23,36 @@ __author__ = 'jaq@google.com (Jamie Wilkinson)'
 import bsddb
 import logging
 import os.path
+import shutil
 import tempfile
 import time
 import unittest
-import pmock
+import sys
+
+import mox
 
 from nss_cache import error
-from nss_cache import maps
-from nss_cache.caches import nssdb
 
-logging.disable(logging.CRITICAL)
+from nss_cache.caches import nssdb
+from nss_cache.maps import group
+from nss_cache.maps import passwd
+from nss_cache.maps import shadow
 
 
 class TestSkipped(Exception):
   """Exception to raise if a test cannot be run."""
 
 
-class TestNssDbPasswdHandler(pmock.MockTestCase):
+class TestNssDbPasswdHandler(mox.MoxTestBase):
 
   def setUp(self):
+    super(TestNssDbPasswdHandler, self).setUp()
     self.workdir = tempfile.mkdtemp()
 
   def tearDown(self):
+    super(TestNssDbPasswdHandler, self).tearDown()
     # remove the test working directory
-    os.rmdir(self.workdir)
+    shutil.rmtree(self.workdir)
 
   def testConvertValueToMapEntry(self):
     ent = 'foo:x:1000:1001:bar:/:/bin/sh'
@@ -72,24 +78,13 @@ class TestNssDbPasswdHandler(pmock.MockTestCase):
   def testNssDbPasswdHandlerWriteData(self):
     entry_string = 'foo:x:1000:1000:foo:/:/bin/sh'
 
-    makedb_stdin = self.mock()
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('.foo %s\n' % entry_string))\
-                  .id('write #1')
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('=1000 %s\n' % entry_string))\
-                  .id('write #2')\
-                  .after('write #1')
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('00 %s\n' % entry_string))\
-                  .id('write #3')\
-                  .after('write #2')
+    makedb_stdin = self.mox.CreateMock(sys.stdin)
+    makedb_stdin.write('.foo %s\n' % entry_string)
+    makedb_stdin.write('=1000 %s\n' % entry_string)
+    makedb_stdin.write('00 %s\n' % entry_string)
 
-    passwd_map = maps.PasswdMap()
-    passwd_map_entry = maps.PasswdMapEntry()
+    passwd_map = passwd.PasswdMap()
+    passwd_map_entry = passwd.PasswdMapEntry()
     passwd_map_entry.name = 'foo'
     passwd_map_entry.uid = 1000
     passwd_map_entry.gid = 1000
@@ -102,42 +97,25 @@ class TestNssDbPasswdHandler(pmock.MockTestCase):
     writer = nssdb.NssDbPasswdHandler({'makedb': '/bin/false',
                                        'dir': '/tmp'})
 
+    self.mox.ReplayAll()
+
     writer.WriteData(makedb_stdin, passwd_map_entry, 0)
 
   def testNssDbPasswdHandlerWrite(self):
     ent = 'foo:x:1000:1000:foo:/:/bin/sh'
 
-    makedb_stdin = self.mock()
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('.foo %s\n' % ent))\
-                  .id('write #1')
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('=1000 %s\n' % ent))\
-                  .id('write #2')\
-                  .after('write #1')
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('00 %s\n' % ent))\
-                  .id('write #3')\
-                  .after('write #2')
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .method('close')\
-                  .after('write #3')
+    makedb_stdin = self.mox.CreateMock(sys.stdin)
+    makedb_stdin.write('.foo %s\n' % ent)
+    makedb_stdin.write('=1000 %s\n' % ent)
+    makedb_stdin.write('00 %s\n' % ent)
+    makedb_stdin.close()
 
-    makedb_stdout = self.mock()
-    makedb_stdout\
-                   .expects(pmock.once())\
-                   .read()\
-                   .will(pmock.return_value(''))
-    makedb_stdout\
-                   .expects(pmock.once())\
-                   .method('close')
+    makedb_stdout = self.mox.CreateMock(sys.stdout)
+    makedb_stdout.read().AndReturn('')
+    makedb_stdout.close()
 
-    m = maps.PasswdMap()
-    pw = maps.PasswdMapEntry()
+    m = passwd.PasswdMap()
+    pw = passwd.PasswdMapEntry()
     pw.name = 'foo'
     pw.uid = 1000
     pw.gid = 1000
@@ -153,18 +131,21 @@ class TestNssDbPasswdHandler(pmock.MockTestCase):
         return 0
 
       def poll(self):
-        return -1
+        return None
 
     def SpawnMakeDb():
       makedb = MakeDbDummy()
-      makedb.tochild = makedb_stdin
-      makedb.fromchild = makedb_stdout
+      makedb.stdin = makedb_stdin
+      makedb.stdout = makedb_stdout
       return makedb
 
     writer = nssdb.NssDbPasswdHandler({'makedb': '/usr/bin/makedb',
                                        'dir': self.workdir})
+
     writer._SpawnMakeDb = SpawnMakeDb
 
+    self.mox.ReplayAll()
+
     writer.Write(m)
 
     tmppasswd = os.path.join(self.workdir, 'passwd.db')
@@ -177,8 +158,8 @@ class TestNssDbPasswdHandler(pmock.MockTestCase):
     if not os.path.exists('/usr/bin/makedb'):
       raise TestSkipped('no /usr/bin/makedb')
     # create a map
-    m = maps.PasswdMap()
-    e = maps.PasswdMapEntry()
+    m = passwd.PasswdMap()
+    e = passwd.PasswdMapEntry()
     e.name = 'foo'
     e.uid = 1000
     e.gid = 2000
@@ -188,14 +169,14 @@ class TestNssDbPasswdHandler(pmock.MockTestCase):
                                         'makedb': '/usr/bin/makedb'})
     written = updater.Write(m)
 
-    self.failUnless(os.path.exists(updater.cache_filename),
+    self.failUnless(os.path.exists(updater.temp_cache_filename),
                     'updater.Write() did not create a file')
 
     retval = updater.Verify(written)
 
     self.failUnlessEqual(True, retval)
 
-    os.unlink(updater.cache_filename)
+    os.unlink(updater.temp_cache_filename)
 
   def testVerifyFailure(self):
     # Can't test if no makedb
@@ -211,8 +192,8 @@ class TestNssDbPasswdHandler(pmock.MockTestCase):
     fltr = TestFilter()
     logging.getLogger('NssDbPasswdHandler').addFilter(fltr)
     # create a map
-    m = maps.PasswdMap()
-    e = maps.PasswdMapEntry()
+    m = passwd.PasswdMap()
+    e = passwd.PasswdMapEntry()
     e.name = 'foo'
     e.uid = 1000
     e.gid = 2000
@@ -222,11 +203,11 @@ class TestNssDbPasswdHandler(pmock.MockTestCase):
                                         'makedb': '/usr/bin/makedb'})
     written = updater.Write(m)
 
-    self.failUnless(os.path.exists(updater.cache_filename),
+    self.failUnless(os.path.exists(updater.temp_cache_filename),
                     'updater.Write() did not create a file')
 
     # change the cache
-    db = bsddb.btopen(updater.cache_filename)
+    db = bsddb.btopen(updater.temp_cache_filename)
     del db[db.first()[0]]
     db.sync()
     db.close()
@@ -234,16 +215,16 @@ class TestNssDbPasswdHandler(pmock.MockTestCase):
     retval = updater.Verify(written)
 
     self.failUnlessEqual(False, retval)
-    self.failIf(os.path.exists(os.path.join(updater.cache_filename)))
+    self.failIf(os.path.exists(os.path.join(updater.temp_cache_filename)))
     # no longer hide this message
     logging.getLogger('NssDbPasswdHandler').removeFilter(fltr)
 
   def testVerifyEmptyMap(self):
     updater = nssdb.NssDbPasswdHandler({'dir': self.workdir})
     # create a temp file, clag it into the updater object
-    (_, temp_filename) = tempfile.mkstemp(prefix='nsscache',
+    (_, temp_filename) = tempfile.mkstemp(prefix='nsscache-nssdb_test',
                                           dir=self.workdir)
-    updater.cache_filename = temp_filename
+    updater.temp_cache_filename = temp_filename
     # make it empty
     db = bsddb.btopen(temp_filename, 'w')
     self.assertEqual(0, len(db))
@@ -253,14 +234,16 @@ class TestNssDbPasswdHandler(pmock.MockTestCase):
     os.unlink(temp_filename)
 
 
-class TestNssDbGroupHandler(pmock.MockTestCase):
+class TestNssDbGroupHandler(mox.MoxTestBase):
 
   def setUp(self):
+    super(TestNssDbGroupHandler, self).setUp()
     self.workdir = tempfile.mkdtemp()
 
   def tearDown(self):
+    super(TestNssDbGroupHandler, self).tearDown()
     # remove the test working directory
-    os.rmdir(self.workdir)
+    shutil.rmtree(self.workdir)
 
   def testConvertValueToMapEntry(self):
     ent = 'foo:x:1000:bar'
@@ -283,24 +266,13 @@ class TestNssDbGroupHandler(pmock.MockTestCase):
   def testNssDbGroupHandlerWriteData(self):
     ent = 'foo:x:1000:bar'
 
-    makedb_stdin = self.mock()
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('.foo %s\n' % ent))\
-                  .id('write #1')
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('=1000 %s\n' % ent))\
-                  .id('write #2')\
-                  .after('write #1')
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('00 %s\n' % ent))\
-                  .id('write #3')\
-                  .after('write #2')
+    makedb_stdin = self.mox.CreateMock(sys.stdin)
+    makedb_stdin.write('.foo %s\n' % ent)
+    makedb_stdin.write('=1000 %s\n' % ent)
+    makedb_stdin.write('00 %s\n' % ent)
 
-    m = maps.GroupMap()
-    g = maps.GroupMapEntry()
+    m = group.GroupMap()
+    g = group.GroupMapEntry()
     g.name = 'foo'
     g.gid = 1000
     g.passwd = 'x'
@@ -311,42 +283,25 @@ class TestNssDbGroupHandler(pmock.MockTestCase):
     writer = nssdb.NssDbGroupHandler({'makedb': '/bin/false',
                                       'dir': '/tmp'})
 
+    self.mox.ReplayAll()
+
     writer.WriteData(makedb_stdin, g, 0)
 
   def testNssDbGroupHandlerWrite(self):
     ent = 'foo:x:1000:bar'
 
-    makedb_stdin = self.mock()
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('.foo %s\n' % ent))\
-                  .id('write #1')
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('=1000 %s\n' % ent))\
-                  .id('write #2')\
-                  .after('write #1')
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('00 %s\n' % ent))\
-                  .id('write #3')\
-                  .after('write #2')
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .method('close')\
-                  .after('write #3')
+    makedb_stdin = self.mox.CreateMock(sys.stdin)
+    makedb_stdin.write('.foo %s\n' % ent)
+    makedb_stdin.write('=1000 %s\n' % ent)
+    makedb_stdin.write('00 %s\n' % ent)
+    makedb_stdin.close()
 
-    makedb_stdout = self.mock()
-    makedb_stdout\
-                   .expects(pmock.once())\
-                   .read()\
-                   .will(pmock.return_value(''))
-    makedb_stdout\
-                   .expects(pmock.once())\
-                   .method('close')
+    makedb_stdout = self.mox.CreateMock(sys.stdout)
+    makedb_stdout.read().AndReturn('')
+    makedb_stdout.close()
 
-    m = maps.GroupMap()
-    g = maps.GroupMapEntry()
+    m = group.GroupMap()
+    g = group.GroupMapEntry()
     g.name = 'foo'
     g.gid = 1000
     g.passwd = 'x'
@@ -359,17 +314,18 @@ class TestNssDbGroupHandler(pmock.MockTestCase):
         return 0
 
       def poll(self):
-        return -1
+        return None
 
     def SpawnMakeDb():
       makedb = MakeDbDummy()
-      makedb.tochild = makedb_stdin
-      makedb.fromchild = makedb_stdout
+      makedb.stdin = makedb_stdin
+      makedb.stdout = makedb_stdout
       return makedb
 
     writer = nssdb.NssDbGroupHandler({'makedb': '/usr/bin/makedb',
                                       'dir': self.workdir})
     writer._SpawnMakeDb = SpawnMakeDb
+    self.mox.ReplayAll()
 
     writer.Write(m)
 
@@ -383,8 +339,8 @@ class TestNssDbGroupHandler(pmock.MockTestCase):
     if not os.path.exists('/usr/bin/makedb'):
       raise TestSkipped('no /usr/bin/makedb')
     # create a map
-    m = maps.GroupMap()
-    e = maps.GroupMapEntry()
+    m = group.GroupMap()
+    e = group.GroupMapEntry()
     e.name = 'foo'
     e.gid = 2000
     self.failUnless(m.Add(e))
@@ -393,13 +349,13 @@ class TestNssDbGroupHandler(pmock.MockTestCase):
                                        'makedb': '/usr/bin/makedb'})
     written = updater.Write(m)
 
-    self.failUnless(os.path.exists(updater.cache_filename),
+    self.failUnless(os.path.exists(updater.temp_cache_filename),
                     'updater.Write() did not create a file')
 
     retval = updater.Verify(written)
 
     self.failUnlessEqual(True, retval)
-    os.unlink(updater.cache_filename)
+    os.unlink(updater.temp_cache_filename)
 
   def testVerifyFailure(self):
     # Can't test if no makedb
@@ -415,8 +371,8 @@ class TestNssDbGroupHandler(pmock.MockTestCase):
     logging.getLogger('NssDbGroupHandler').addFilter(fltr)
 
     # create a map
-    m = maps.GroupMap()
-    e = maps.GroupMapEntry()
+    m = group.GroupMap()
+    e = group.GroupMapEntry()
     e.name = 'foo'
     e.gid = 2000
     self.failUnless(m.Add(e))
@@ -425,11 +381,11 @@ class TestNssDbGroupHandler(pmock.MockTestCase):
                                        'makedb': '/usr/bin/makedb'})
     written = updater.Write(m)
 
-    self.failUnless(os.path.exists(updater.cache_filename),
+    self.failUnless(os.path.exists(updater.temp_cache_filename),
                     'updater.Write() did not create a file')
 
     # change the cache
-    db = bsddb.btopen(updater.cache_filename)
+    db = bsddb.btopen(updater.temp_cache_filename)
     del db[db.first()[0]]
     db.sync()
     db.close()
@@ -437,19 +393,21 @@ class TestNssDbGroupHandler(pmock.MockTestCase):
     retval = updater.Verify(written)
 
     self.failUnlessEqual(False, retval)
-    self.failIf(os.path.exists(os.path.join(updater.cache_filename)))
+    self.failIf(os.path.exists(os.path.join(updater.temp_cache_filename)))
     # no longer hide this message
     logging.getLogger('NssDbGroupHandler').removeFilter(fltr)
 
 
-class TestNssDbShadowHandler(pmock.MockTestCase):
+class TestNssDbShadowHandler(mox.MoxTestBase):
 
   def setUp(self):
+    super(TestNssDbShadowHandler, self).setUp()
     self.workdir = tempfile.mkdtemp()
 
   def tearDown(self):
+    super(TestNssDbShadowHandler, self).tearDown()
     # remove the test working directory
-    os.rmdir(self.workdir)
+    shutil.rmtree(self.workdir)
 
   def testConvertValueToMapEntry(self):
     ent = 'foo:*:::::::0'
@@ -471,19 +429,12 @@ class TestNssDbShadowHandler(pmock.MockTestCase):
   def testNssDbShadowHandlerWriteData(self):
     ent = 'foo:!!:::::::0'
 
-    makedb_stdin = self.mock()
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('.foo %s\n' % ent))\
-                  .id('write #1')
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('00 %s\n' % ent))\
-                  .id('write #2')\
-                  .after('write #1')
+    makedb_stdin = self.mox.CreateMock(sys.stdin)
+    makedb_stdin.write('.foo %s\n' % ent)
+    makedb_stdin.write('00 %s\n' % ent)
 
-    m = maps.ShadowMap()
-    s = maps.ShadowMapEntry()
+    m = shadow.ShadowMap()
+    s = shadow.ShadowMapEntry()
     s.name = 'foo'
 
     self.failUnless(m.Add(s))
@@ -491,34 +442,24 @@ class TestNssDbShadowHandler(pmock.MockTestCase):
     writer = nssdb.NssDbShadowHandler({'makedb': '/bin/false',
                                        'dir': '/tmp'})
 
+    self.mox.ReplayAll()
+
     writer.WriteData(makedb_stdin, s, 0)
 
   def testNssDbShadowHandlerWrite(self):
     ent = 'foo:*:::::::0'
 
-    makedb_stdin = self.mock()
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('.foo %s\n' % ent))\
-                  .id('write #1')
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .write(pmock.eq('00 %s\n' % ent))\
-                  .id('write #2')\
-                  .after('write #1')
-    makedb_stdin\
-                  .expects(pmock.once())\
-                  .method('close')\
-                  .after('write #2')
+    makedb_stdin = self.mox.CreateMock(sys.stdin)
+    makedb_stdin.write('.foo %s\n' % ent)
+    makedb_stdin.write('00 %s\n' % ent)
+    makedb_stdin.close()
 
-    makedb_stdout = self.mock()
-    makedb_stdout.expects(pmock.once()).read().will(pmock.return_value(''))
-    makedb_stdout\
-                   .expects(pmock.once())\
-                   .method('close')
+    makedb_stdout = self.mox.CreateMock(sys.stdout)
+    makedb_stdout.read().AndReturn('')
+    makedb_stdout.close()
 
-    m = maps.ShadowMap()
-    s = maps.ShadowMapEntry()
+    m = shadow.ShadowMap()
+    s = shadow.ShadowMapEntry()
     s.name = 'foo'
     s.passwd = '*'
     s.Verify()
@@ -529,17 +470,18 @@ class TestNssDbShadowHandler(pmock.MockTestCase):
         return 0
 
       def poll(self):
-        return -1
+        return None
 
     def SpawnMakeDb():
       makedb = MakeDbDummy()
-      makedb.tochild = makedb_stdin
-      makedb.fromchild = makedb_stdout
+      makedb.stdin = makedb_stdin
+      makedb.stdout = makedb_stdout
       return makedb
 
     writer = nssdb.NssDbShadowHandler({'makedb': '/usr/bin/makedb',
                                        'dir': self.workdir})
     writer._SpawnMakeDb = SpawnMakeDb
+    self.mox.ReplayAll()
 
     writer.Write(m)
 
@@ -552,8 +494,8 @@ class TestNssDbShadowHandler(pmock.MockTestCase):
     # Can't test if no makedb
     if not os.path.exists('/usr/bin/makedb'):
       raise TestSkipped('no /usr/bin/makedb')
-    m = maps.ShadowMap()
-    s = maps.ShadowMapEntry()
+    m = shadow.ShadowMap()
+    s = shadow.ShadowMapEntry()
     s.name = 'foo'
     self.failUnless(m.Add(s))
 
@@ -561,13 +503,13 @@ class TestNssDbShadowHandler(pmock.MockTestCase):
                                         'makedb': '/usr/bin/makedb'})
     written = updater.Write(m)
 
-    self.failUnless(os.path.exists(updater.cache_filename),
+    self.failUnless(os.path.exists(updater.temp_cache_filename),
                     'updater.Write() did not create a file')
 
     retval = updater.Verify(written)
 
     self.failUnlessEqual(True, retval)
-    os.unlink(updater.cache_filename)
+    os.unlink(updater.temp_cache_filename)
 
   def testVerifyFailure(self):
 
@@ -585,8 +527,8 @@ class TestNssDbShadowHandler(pmock.MockTestCase):
     logging.getLogger('NssDbShadowHandler').addFilter(fltr)
 
     # create a map
-    m = maps.ShadowMap()
-    s = maps.ShadowMapEntry()
+    m = shadow.ShadowMap()
+    s = shadow.ShadowMapEntry()
     s.name = 'foo'
     self.failUnless(m.Add(s))
 
@@ -594,11 +536,11 @@ class TestNssDbShadowHandler(pmock.MockTestCase):
                                         'makedb': '/usr/bin/makedb'})
     written = updater.Write(m)
 
-    self.failUnless(os.path.exists(updater.cache_filename),
+    self.failUnless(os.path.exists(updater.temp_cache_filename),
                     'updater.Write() did not create a file')
 
     # change the cache
-    db = bsddb.btopen(updater.cache_filename)
+    db = bsddb.btopen(updater.temp_cache_filename)
     del db[db.first()[0]]
     db.sync()
     db.close()
@@ -606,7 +548,7 @@ class TestNssDbShadowHandler(pmock.MockTestCase):
     retval = updater.Verify(written)
 
     self.failUnlessEqual(False, retval)
-    self.failIf(os.path.exists(os.path.join(updater.cache_filename)))
+    self.failIf(os.path.exists(os.path.join(updater.temp_cache_filename)))
     # no longer hide this message
     logging.getLogger('NssDbShadowHandler').removeFilter(fltr)
 
@@ -614,18 +556,19 @@ class TestNssDbShadowHandler(pmock.MockTestCase):
 class TestNssDbCache(unittest.TestCase):
 
   def setUp(self):
+    super(TestNssDbCache, self).setUp()
     self.workdir = tempfile.mkdtemp()
 
   def tearDown(self):
-    # remove the test working directory
-    os.rmdir(self.workdir)
+    super(TestNssDbCache, self).tearDown()
+    shutil.rmtree(self.workdir)
 
   def testWriteTestBdb(self):
     # Can't test if no makedb
     if not os.path.exists('/usr/bin/makedb'):
       raise TestSkipped('no /usr/bin/makedb')
-    data = maps.PasswdMap()
-    pw = maps.PasswdMapEntry()
+    data = passwd.PasswdMap()
+    pw = passwd.PasswdMapEntry()
     pw.name = 'foo'
     pw.passwd = 'x'
     pw.uid = 1000
@@ -640,12 +583,11 @@ class TestNssDbCache(unittest.TestCase):
     cache = nssdb.NssDbPasswdHandler(dummy_config)
 
     written = cache.Write(data)
-
     self.assertTrue('.foo' in written)
     self.assertTrue('=1000' in written)
 
     # perform test
-    db = bsddb.btopen(cache.cache_filename, 'r')
+    db = bsddb.btopen(cache.temp_cache_filename, 'r')
 
     self.assertEqual(3, len(db.keys()))
     self.failUnless('.foo' in db.keys())
@@ -660,7 +602,7 @@ class TestNssDbCache(unittest.TestCase):
     self.assertEqual(db['=1000'], d)
 
     # tear down
-    os.unlink(cache.cache_filename)
+    os.unlink(cache.temp_cache_filename)
 
   def testLoadBdbCacheFile(self):
     pass_file = os.path.join(self.workdir, 'passwd.db')
@@ -731,7 +673,7 @@ class TestNssDbCache(unittest.TestCase):
     db.close()
     cache = nssdb.NssDbPasswdHandler({'dir': self.workdir})
     cache_map = cache.GetMap()
-    self.assertEquals(False, cache_map.Merge(maps.PasswdMap()))
+    self.assertEquals(False, cache_map.Merge(passwd.PasswdMap()))
     os.unlink(update_ts_filename)
     os.unlink(db_filename)
 
