@@ -1,4 +1,4 @@
-#!/usr/bin/python2.4
+#!/usr/bin/python
 #
 # Copyright 2007 Google Inc.
 #
@@ -21,27 +21,26 @@
 __author__ = ('jaq@google.com (Jamie Wilkinson)',
               'vasilios@google.com (Vasilios Hoffman)')
 
+
+import calendar
 import inspect
 import logging
 import optparse
 import os
+import shutil
 import StringIO
+import tempfile
 import time
 
-from nss_cache import caches
 from nss_cache import config
 from nss_cache import error
 from nss_cache import lock
 from nss_cache import nss
-from nss_cache import sources
-from nss_cache import update
 
-# python2.3 has no builtin set() class
-try:
-  Set = set
-except NameError:
-  import sets
-  Set = sets.Set
+from nss_cache.caches import cache_factory
+from nss_cache.sources import source_factory
+from nss_cache.update import map_updater
+from nss_cache.update import files_updater
 
 
 class Command(object):
@@ -255,37 +254,95 @@ class Update(Command):
       self.log.error('Failed to acquire lock, aborting!')
       return self.ERR_LOCK
 
+    retval = 0
     for map_name in conf.maps:
-      self.log.debug('operating on map: %s', map_name)
-
-      cache_options = conf.options[map_name].cache
-      source_options = conf.options[map_name].source
-
-      source = sources.base.Create(source_options)
-
-      if map_name == config.MAP_AUTOMOUNT:
-        updater = update.AutomountUpdater(map_name, conf.timestamp_dir,
-                                          cache_options)
-      else:
-        updater = update.SingleMapUpdater(map_name, conf.timestamp_dir,
-                                          cache_options)
+      if map_name not in conf.options:
+        self.log.error('No such map name defined in config: %s', map_name)
+        return 1
 
       if incremental:
         self.log.info('Updating and verifying %s cache.', map_name)
       else:
         self.log.info('Rebuilding and verifying %s cache.', map_name)
 
+      cache_options = conf.options[map_name].cache
+      source_options = conf.options[map_name].source
+
+      # Change into the target directory.
+      # Sources such as zsync handle their temporary files badly, so we
+      # want to be in the same location that the destination file will
+      # exist in, so that the atomic rename occurs in the same
+      # filesystem.
+      # In addition, we create a tempdir below this dir to work in, because
+      # zsync's librcksum sometimes leaves temp files around, and we don't
+      # want to leave file turds around /etc.
+      # We save and restore the directory here as each cache can define its own
+      # output directory.
+      # Finally, relative paths in the config are treated as relative to the
+      # startup directory, but we convewrt them to absolute paths so that future
+      # temp dirs do not mess with our output routines.
+      old_cwd = os.getcwd()
+      tempdir = tempfile.mkdtemp(dir=cache_options['dir'],
+                                 prefix='nsscache-%s-' % map_name)
+      if not os.path.isabs(cache_options['dir']):
+        cache_options['dir'] = os.path.abspath(cache_options['dir'])
+      if not os.path.isabs(conf.timestamp_dir):
+        conf.timestamp_dir = os.path.abspath(conf.timestamp_dir)
+      if not os.path.isabs(tempdir):
+        tempdir = os.path.abspath(tempdir)
+      os.chdir(tempdir)
+      # End chdir dirty hack.
+
       try:
-        retval = updater.UpdateFromSource(source, incremental=incremental, force_write=force_write)
+        source = source_factory.Create(source_options)
+
+        updater = self._Updater(map_name, source, cache_options, conf)
+
+        if incremental:
+          self.log.info('Updating and verifying %s cache.', map_name)
+        else:
+          self.log.info('Rebuilding and verifying %s cache.', map_name)
+
+        retval = updater.UpdateFromSource(source, incremental=incremental,
+                                          force_write=force_write)
       except error.PermissionDenied:
         self.log.error('Permission denied: could not update map %r.  Aborting',
                        map_name)
-        retval = 1
+        retval += 1
+      except (error.EmptyMap, error.InvalidMap), e:
+        self.log.error(e)
+        retval += 1
+      except error.InvalidMerge, e:
+        self.log.warn('Could not merge map %r: %s.  Skipping.',
+                       map_name, e)
+      finally:
+        # Start chdir cleanup
+        os.chdir(old_cwd)
+        print "udpate rmtree %s" % tempdir
+        shutil.rmtree(tempdir)
+        # End chdir cleanup
 
-      if retval:
-        return retval
+    return retval
 
-    return 0
+  def _Updater(self, map_name, source, cache_options, conf):
+    # Bit ugly. This just checks the class attribute UPDATER
+    # to determine which type of updater the source uses. At the moment
+    # there's only two, so not a huge deal. If we add another we should
+    # refactor though.
+    if hasattr(source, 'UPDATER') and source.UPDATER == config.UPDATER_FILE:
+      if map_name == config.MAP_AUTOMOUNT:
+        return files_updater.FileAutomountUpdater(map_name, conf.timestamp_dir,
+                                                     cache_options)
+      else:
+        return files_updater.FileMapUpdater(map_name, conf.timestamp_dir,
+                                               cache_options)
+    else:
+      if map_name == config.MAP_AUTOMOUNT:
+        return map_updater.AutomountUpdater(map_name, conf.timestamp_dir,
+                                               cache_options)
+      else:
+        return map_updater.MapUpdater(map_name, conf.timestamp_dir,
+                                         cache_options)
 
 
 class Verify(Command):
@@ -363,7 +420,7 @@ class Verify(Command):
       # The netgroup map does not have an enumerator,
       # to test this we'd have to loop over the loaded cache map
       # and verify each entry is retrievable via getent directly.
-      # TODO: apply fix from comment to allow for netgroup checking
+      # TODO(blaed): apply fix from comment to allow for netgroup checking
       if map_name == config.MAP_NETGROUP:
         self.log.info(('The netgroup map does not support enumeration, '
                        'skipping.'))
@@ -385,7 +442,7 @@ class Verify(Command):
       self.log.debug('built NSS map of %d entries', len(nss_map))
 
       cache_options = conf.options[map_name].cache
-      cache = caches.base.Create(cache_options, map_name)
+      cache = cache_factory.Create(cache_options, map_name)
 
       try:
         cache_map = cache.GetMap()
@@ -415,7 +472,7 @@ class Verify(Command):
 
   def VerifySources(self, conf):
     """Verify each possible source and return the appropriate retval."""
-    possible_sources = Set()
+    possible_sources = set()
     retval = 0
 
     for map_name in conf.maps:
@@ -425,7 +482,7 @@ class Verify(Command):
       for map_name in possible_sources:
         source_options = conf.options[map_name].source
         try:
-          source = sources.base.Create(source_options)
+          source = source_factory.Create(source_options)
         except error.SourceUnavailable, e:
           self.log.debug('map %s dumps source error %s', map_name, e)
           self.log.error('Map %s is unvavailable!', map_name)
@@ -539,18 +596,21 @@ class Status(Command):
 
   def __init__(self):
     super(Status, self).__init__()
-    self.parser.add_option('--values-only',
-                           action='store_true',
-                           help='show values only, no labels',
-                           dest='values_only', default=False)
-    self.parser.add_option('--stat',
-                           action='store',
-                           help='only show status field FIELD',
-                           metavar='FIELD', dest='stat')
     self.parser.add_option('--epoch',
                            action='store_true',
                            help='show timestamps in UNIX epoch time',
                            dest='epoch', default=False)
+    self.parser.add_option('--template',
+                           action='store',
+                           help='Set format for output',
+                           metavar='FORMAT', dest='template',
+                           default='NSS map: %(map)s\n%(key)s: %(value)s')
+    self.parser.add_option('--automount-template',
+                           action='store',
+                           help='Set format for automount output',
+                           metavar='FORMAT', dest='automount_template',
+                           default=('NSS map: %(map)s\nAutomount map: '
+                                    '%(automount)s\n%(key)s: %(value)s'))
 
   def Run(self, conf, args):
     """Run the Status command.
@@ -574,106 +634,81 @@ class Status(Command):
       self.log.info('Setting configured maps to %s', options.maps)
       conf.maps = options.maps
 
-    template = self.GetOutputTemplate(options.values_only,
-                                      options.stat)
-
     for map_name in conf.maps:
-      if len(conf.maps) > 1 and not options.values_only:
-        print 'NSS map:', map_name
       # Hardcoded to support the two-tier structure of automount maps
       if map_name == config.MAP_AUTOMOUNT:
         value_list = self.GetAutomountMapMetadata(conf, epoch=options.epoch)
+        self.log.debug('Value list: %r', value_list)
         for value_dict in value_list:
-          if not options.values_only:
-            print 'Automount map:', value_dict['map']
-          output = template % value_dict
+          self.log.debug('Value dict: %r', value_dict)
+          output = options.automount_template % value_dict
           print output
       else:
-        value_dict = self.GetSingleMapMetadata(map_name, conf,
-                                               epoch=options.epoch)
-        output = template % value_dict
-        print output
+        for value_dict in self.GetSingleMapMetadata(map_name, conf,
+                                                    epoch=options.epoch):
+          self.log.debug('Value dict: %r', value_dict)
+          output = options.template % value_dict
+          print output
 
     return os.EX_OK
 
-  def GetOutputTemplate(self, values_only=False, only_key=None):
-    """Build a template for outputting status information.
-
-    Args:
-      values_only: boolean indicating that the output should not have labels
-      only_key: a list of strings indicating which key/value pairs should be
-        shown, if set to None, then all values are shown.
-
-    Returns:
-      String that can be modified using the string mod operator
-    """
-    # This is a sneaky sneak.  We want to create a bunch of lines for each
-    # template in the same style, so we have a template string that we fill
-    # with each metadata key in the right spot, and possibly the label too.
-    # This then gives back a new template that can have the actual
-    if values_only:
-      key_template = '%%(%(key)s)s'
-    else:
-      key_template = '%(key)s: %%(%(key)s)s'
-
-    # only show these keys
-    if only_key is not None:
-      keys = [only_key]
-    else:
-      keys = ['last-modify-timestamp',
-              'last-update-timestamp']
-
-    # now build the template for the keys we want
-    template = []
-    for key in keys:
-      template.append(key_template % {'key': key})
-
-    return '\n'.join(template)
-
-  def GetSingleMapMetadata(self, map_name, conf, automount_info=None,
+  def GetSingleMapMetadata(self, map_name, conf, automount_mountpoint=None,
                            epoch=False):
     """Return metadata from map specified.
 
     Args:
       map_name: name of map to extract data from
       conf: a config.Config object
-      automount_info: information necessary for automount maps
+      automount_mountpoint: information necessary for automount maps
       epoch: return times as an integer epoch (time_t) instead of a
         human readable name
 
     Returns:
-      a dict of metadata key/value pairs
+      a list of dicts of metadata key/value pairs
     """
     cache_options = conf.options[map_name].cache
 
-    updater = update.SingleMapUpdater(map_name, conf.timestamp_dir,
-                                      cache_options, automount_info)
+    updater = map_updater.MapUpdater(map_name, conf.timestamp_dir,
+                                     cache_options, automount_mountpoint)
 
+    modify_dict = {'key': 'last-modify-timestamp',
+                   'map': map_name}
+    update_dict = {'key': 'last-update-timestamp',
+                   'map': map_name}
     if map_name == config.MAP_AUTOMOUNT:
       # have to find out *which* automount map from a cache object!
-      cache = caches.base.Create(cache_options, config.MAP_AUTOMOUNT,
-                                 automount_info=automount_info)
-      value_dict = {'map': cache.GetMapLocation()}
-    else:
-      value_dict = {'map': map_name}
+      cache = cache_factory.Create(cache_options, config.MAP_AUTOMOUNT,
+                                   automount_mountpoint=automount_mountpoint)
+      automount = cache.GetMapLocation()
+      modify_dict['automount'] = automount
+      update_dict['automount'] = automount
 
     last_modify_timestamp = updater.GetModifyTimestamp()
     last_update_timestamp = updater.GetUpdateTimestamp()
 
     if not epoch:
       if last_modify_timestamp is not None:
-        last_modify_timestamp = time.ctime(last_modify_timestamp)
+        last_modify_timestamp = time.asctime(last_modify_timestamp)
       else:
         last_modify_timestamp = 'Unknown'
       if last_update_timestamp is not None:
-        last_update_timestamp = time.ctime(last_update_timestamp)
+        last_update_timestamp = time.asctime(last_update_timestamp)
       else:
         last_update_timestamp = 'Unknown'
+    else:
+      if last_modify_timestamp is not None:
+        last_modify_timestamp = calendar.timegm(last_modify_timestamp)
+      else:
+        last_modify_timestamp = 0
+      if last_update_timestamp is not None:
+        last_update_timestamp = calendar.timegm(last_update_timestamp)
+      else:
+        last_update_timestamp = 0
 
-    value_dict['last-modify-timestamp'] = last_modify_timestamp
-    value_dict['last-update-timestamp'] = last_update_timestamp
+    modify_dict['value'] = last_modify_timestamp
+    update_dict['value'] = last_update_timestamp
 
-    return value_dict
+    return [modify_dict, update_dict]
 
   def GetAutomountMapMetadata(self, conf, epoch=False):
     """Return status of automount master map and all listed automount maps.
@@ -693,22 +728,21 @@ class Status(Command):
     cache_options = conf.options[map_name].cache
     value_list = []
 
-    # get the value_dict for the master map, note that automount_info=None
+    # get the value_dict for the master map, note that automount_mountpoint=None
     # defaults to the master map!
-    value_dict = self.GetSingleMapMetadata(map_name, conf, automount_info=None,
-                                           epoch=epoch)
-    value_list.append(value_dict)
+    values = self.GetSingleMapMetadata(
+        map_name, conf, automount_mountpoint=None, epoch=epoch)
+    value_list.extend(values)
 
     # now get the contents of the master map, and get the status for each map
     # we find
-    cache = caches.base.Create(cache_options, config.MAP_AUTOMOUNT,
-                               automount_info=None)
+    cache = cache_factory.Create(cache_options, config.MAP_AUTOMOUNT,
+                                 automount_mountpoint=None)
     master_map = cache.GetMap()
 
     for map_entry in master_map:
-      value_dict = self.GetSingleMapMetadata(map_name, conf,
-                                             automount_info=map_entry.key,
-                                             epoch=epoch)
-      value_list.append(value_dict)
-
+      values = self.GetSingleMapMetadata(map_name, conf,
+                                         automount_mountpoint=map_entry.key,
+                                         epoch=epoch)
+      value_list.extend(values)
     return value_list
